/* PyAuthD_NSS.h
   -------------

   Constants that are required for the glibc NSS binding of PyAuthD.

   Copyright (C) 2005, Heiko Wundram.
   Released under a BSD-style license, see LICENSE for details.

$LastChangedBy: modelnine $
$LastChangedRevision: 76 $
$LastChangedDate: 2005-10-24 01:55:20 +0200 (pon, 24 paź 2005) $
*/

#ifndef __INCLUDE_PYAUTHD_NSS_H
#define __INCLUDE_PYAUTHD_NSS_H

/* Type definitions */

#define NSS_STATUS enum nss_status

/* Constants which define what kind of functions may be called. */

#include "PyAuthD_constants.h"

/* NSS functions. */

NSS_STATUS _nss_PyAuthD_getpwuid_r(uid_t uid, struct passwd *pw, char *buffer,
				   size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_getpwnam_r(const char *name, struct passwd *pw,
				   char *buffer, size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_setpwent();
NSS_STATUS _nss_PyAuthD_endpwent();
NSS_STATUS _nss_PyAuthD_getpwent_r(struct passwd *pw, char *buffer,
				   size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_getspuid_r(uid_t uid, struct spwd *pw, char *buffer,
				   size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_getspnam_r(const char *name, struct spwd *pw,
				   char *buffer, size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_setspent();
NSS_STATUS _nss_PyAuthD_endspent();
NSS_STATUS _nss_PyAuthD_getspent_r(struct spwd *pw, char *buffer,
				   size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_getgrgid_r(gid_t gid, struct group *gr, char *buffer,
				   size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_getgrnam_r(const char *name, struct group *gr,
				   char *buffer, size_t buflen, int *errnop);
NSS_STATUS _nss_PyAuthD_setgrent();
NSS_STATUS _nss_PyAuthD_endgrent();
NSS_STATUS _nss_PyAuthD_getgrent_r(struct group *gr, char *buffer,
				   size_t buflen, int *errnop);

/* Macros for NSS */

#define RECV_STRING_FIELD(f,fl) { RECV_VALUE(fl); \
if( fl >= buflen ) goto MEMORY_ERROR; \
buflen -= fl+1; \
f = buffer; \
if( fl && ( recv(sock,(void*)f,fl,0) == -1 ) ) \
goto RECV_ERROR; \
((char*)f)[fl] = '\0'; \
buffer += fl+1; } while (0)

#define RECV_VARSTRING_FIELD(f,fl,vsl) { RECV_VALUE(vsl); \
if( !vsl ) f = NULL; \
else { \
vsl *= sizeof(char*); \
if( vsl > buflen ) goto MEMORY_ERROR; \
f = (char**)buffer; \
buffer += vsl; \
buflen -= vsl; \
while(vsl) { \
vsl -= sizeof(char*); \
RECV_STRING_FIELD(f[vsl],fl); \
}; } } while(0)

#define RECV_PASSWD(pw,dl) { RECV_STRING_FIELD(pw->pw_name,dl); \
RECV_STRING_FIELD(pw->pw_passwd,dl); \
RECV_VALUE(pw->pw_uid); \
RECV_VALUE(pw->pw_gid); \
RECV_STRING_FIELD(pw->pw_gecos,dl); \
RECV_STRING_FIELD(pw->pw_dir,dl); \
RECV_STRING_FIELD(pw->pw_shell,dl); } while (0)

#define RECV_SHADOW(spw,dl) { RECV_STRING_FIELD(spw->sp_namp,dl); \
RECV_STRING_FIELD(spw->sp_pwdp,dl); \
RECV_VALUE(spw->sp_lstchg); \
RECV_VALUE(spw->sp_min); \
RECV_VALUE(spw->sp_max); \
RECV_VALUE(spw->sp_warn); \
RECV_VALUE(spw->sp_inact); \
RECV_VALUE(spw->sp_expire); \
RECV_VALUE(spw->sp_flag); } while (0)

#define RECV_GROUP(gr,dl,vsl) { RECV_STRING_FIELD(gr->gr_name,dl); \
RECV_VALUE(gr->gr_gid); \
RECV_VARSTRING_FIELD(gr->gr_mem,dl,vsl); } while (0)

#endif /* __INCLUDE_PYAUTHD_NSS_H */
