/* PyAuthD_PAM.h
   -------------

   Constants that are required for the PAM binding of PyAuthD.

   Copyright (C) 2005, Heiko Wundram.
   Released under a BSD-style license, see LICENSE for details.

$LastChangedBy: modelnine $
$LastChangedRevision: 76 $
$LastChangedDate: 2005-10-24 01:55:20 +0200 (pon, 24 paź 2005) $
*/

#ifndef __INCLUDE_PYAUTHD_PAM_H
#define __INCLUDE_PYAUTHD_PAM_H

/* Constants which define the methods that may be called on the pam_handle. */

#include "PyAuthD_constants.h"

/* Convenience macro for PAM functions (they all look equal). */

#define DECLARE_FUNCTION(name) PAM_EXTERN int pam_sm_ ## name ( \
pam_handle_t *pamh, int flags, int argc,const char **argv) \
{ \
  int pamsock, pamrv; \
  if( ( pamsock = openPyAuthD(AUTHD_PAM) ) == -1 ) \
    goto SOCKET_ERROR; \
  if( sendFunctionArguments(pamsock,#name,flags,argc,argv) == -1 ) \
    goto SOCKET_ERROR; \
  if( ( pamrv = communicatePam(pamsock,pamh) ) == -1 ) \
    goto SOCKET_ERROR; \
  closePyAuthD(pamsock); \
  return pamrv; \
 SOCKET_ERROR: \
  closePyAuthD(pamsock); \
  return PAM_SERVICE_ERR; \
}

/* PAM functions. */

int sendFunctionArguments(int sock, const char *funcname, int flags, int argc,
			  const char **argv);
int communicatePam(int sock, pam_handle_t *pamh);
void cleanupPam(pam_handle_t *pamh, void *data, int error_status);

#endif /* __INCLUDE_PYAUTHD_PAM_H */
