# -*- coding: iso-8859-15 -*-
"""pyauthd_ppp.py

Implementation of the PPP plugin for the python authentication daemon. The PPP
plugin handles all communication with PyAuthD_PPP.c, the PyAuthD PPP module.

Copyright (C) 2005, Heiko Wundram.
Released under a BSD-style license, see LICENSE for details.
"""

# Versioning.
__author__ = "$LastChangedBy: modelnine $"
__version__ = "$LastChangedRevision: 158 $"
__date__ = "$LastChangedDate: 2005-11-04 20:38:56 +0100 (pią, 04 lis 2005) $"

# Imports.

import socket
import struct
from pyauthd_serv import BaseMixin
from constants import *

# PPP specific stuff.

class PPPMixin(BaseMixin):
    """Mixin class for BasePyAuthDClient which handles incoming PPP events. It
    implements appropriate handling of PPP structures."""

    # Main handling method. This is called by the Clients handle() method.

    def handle_PPP(self):
        # Check whether we are really connected by pppd (root). If not,
        # somebody is fucking with us! TODO
        pass

        # Loop until we get exit signal from PPP.
        while True:
            # Switch over signals and process them.
            comm = self.recvValue()

            if comm == PPP_GET_SECRET:
                user = self.recvData()
                if user is not None:
                    ourname = self.recvData()
                else:
                    ourname = None
                self.handle_getSecret(user,ourname)
                rv = None
            elif comm == PPP_CHOOSE_IP:
                rv = self.__inet_aton(self.handle_chooseIP(
                    self.__inet_ntoa(self.recvValue())))
            elif comm == PPP_CHECK_IP:
                rv = self.handle_checkIP(self.__inet_ntoa(self.recvValue()))
            elif comm == PPP_CHECK_IDLE:
                xmit_idle = self.recvValue()
                if xmit_idle <> -1:
                    rv = self.handle_checkIdle(xmit_idle,self.recvValue())
                else:
                    rv = self.handle_checkIdle(None,None)
            elif comm == PPP_CHECK_HOLDOFF:
                rv = self.handle_checkHoldoff()
            elif comm == PPP_SIGNAL_EXIT:
                break
            else:
                # Find a suitable way to handle notifiers. For now, just
                # close channel and exit.
                arg = self.recvValue()
                rv = 0

            # Send return value; rv is None when we handled a GET_SECRET
            # event.
            if rv is not None:
                self.sendValue(rv)

    # Private methods.

    def __inet_ntoa(self,ip):
        return socket.inet_ntoa(struct.pack("i",ip))

    def __inet_aton(self,ip):
        return struct.unpack("i",socket.inet_aton(ip))[0]

    # Handlers.

    def handle_getSecret(self,user,ourname):
        if user is None:
            # Anonymous authentication, this is a complete NO NO!
            self.sendValue(0)
            return

        # Get user data and return secret. Ignore ourname for now.
        try:
            self.user = user = self.getUser(user)
            self.sendData(user.getPass())
        except KeyError:
            # User wasn't found, return this.
            self.sendData(None)

    def handle_chooseIP(self,ip):
        # Alter IP choice, for now just return IP.
        return ip

    def handle_checkIP(self,ip):
        # Check IP choice, for now just signal okay.
        return 1

    def handle_checkIdle(self,xmit_idle,recv_idle):
        # Check idle times, for now just ignore.
        return 1

    def handle_checkHoldoff(self):
        # Check holdoff of current user. For now, this is just okay.
        return 1
