# -*- coding: iso-8859-15 -*-
"""pyauthd_squirrelmail.py

Implementation of squirrelmail backend handling. This just sends email and
name info over the socket.

Copyright (C) 2005, Heiko Wundram.
Released under a BSD-style license, see LICENSE for details.
"""

# Versioning.
__author__ = "$LastChangedBy: modelnine $"
__version__ = "$LastChangedRevision: 163 $"
__date__ = "$LastChangedDate: 2005-11-10 18:18:48 +0100 (czw, 10 lis 2005) $"

# Our imports.

from pyauthd_serv import BaseMixin
from constants import *

# NSS specific stuff.

class SquirrelmailMixin(BaseMixin):
    """Mixin class for BasePyAuthDClient which handles incoming Squirrelmail
    communication."""

    # Main handling method. This is called by the Clients handle() method.

    def handle_SQUIRRELMAIL(self):
        # Squirrelmail keeps the connection open, able to send more than one
        # comm.
        while True:
            # Get comm info.
            comm = self.recvValue()
            print "Got comm:", comm

            # Switch on comms.
            if comm == SQUIRRELMAIL_GETNAMEALIAS:
                self.handle_getNameAlias(self.recvData())
            elif comm == SQUIRRELMAIL_SETALIAS:
                self.handle_setAlias(self.recvData(),self.recvData())
            elif comm == SQUIRRELMAIL_CLOSE:
                break
            else:
                raise RuntimeError, "Invalid comm from Squirrelmail."

    # Handlers.

    def handle_getNameAlias(self,user):
        print "getNameAlias:", user
        try:
            user = self.getUser(user)
            name, alias = user.getMailInfo()
        except:
            # Huh?! The user was logged in by us but no info available?!
            # Okay, this may actually happen when the user isn't stored in the
            # PyAuthD database but still uses Squirrelmail (IMAP) which
            # authenticates using PAM. If we return "", this just tells
            # retrieveuserdata to use the parameters set without changing them.
            name, alias = "", ""

        # There is no return code for this; private API.
        print "Sending:", name
        self.sendData(name)
        print "Sending:", alias
        self.sendData(alias)

    def handle_setAlias(self,uid,alias):
        try:
            user = self.getUser(user)
            user.setMailInfo(alias)
            self.sendValue(1)
        except Exception, e:
            # User class either wasn't found or denied setting mail info;
            # return false to Squirrelmail. Denying to set mail info may
            # be caused by duplicate alias. This is signaled in the exception.
            self.sendValue(0)
            self.sendData(str(e))
