# -*- coding: iso-8859-15 -*-
"""studauthd_db.py

Student authentication database settings. This file contains implementations
for the 

Copyright (C) 2005, Heiko Wundram.
Released under a BSD-style license, see LICENSE for details.
"""

# Versioning.
__author__ = "$LastChangedBy: modelnine $"
__version__ = "$LastChangedRevision: 211 $"
__date__ = "$LastChangedDate: 2005-12-15 08:06:01 +0100 (czw, 15 gru 2005) $"

# Imports.

import config
import os
import smtplib
import sqlobject

# We need NSS special classes to implement access to Passwd and Shadow.

import pyauthd_nss

# Implement database access using SQLObject.

__connection__ = sqlobject.connectionForURI("mysql://"+config.DB_USER+":"+
                                            config.DB_PASS+"@localhost/"+
                                            config.DB_DB)

class user(sqlobject.SQLObject):

    # Name.
    vorname = sqlobject.StringCol(length=255)
    nachname = sqlobject.StringCol(length=255)

    # Persoenliche Informationen.
    gebdat = sqlobject.DateCol()
    semester = sqlobject.IntCol()
    fach = sqlobject.ForeignKey("fach")

    # Account Informationen.
    matnr = sqlobject.IntCol()
    aliases = sqlobject.MultipleJoin("alias")
    passwd = sqlobject.StringCol(length=255)

    # Alias logic.
    def __getAlias(self):
        # Search for alias.
        curalias = None
        for ralias in self.aliases:
            curalias = ralias.alias
            if ralias.aliastype == 1:
                # Type 1 is default; there are currently just type 1 aliases.
                break

        # Curalias is last found or type 1.
        return curalias

    alias = property(__getAlias,None,None,"Get the current effective alias.")

class fach(sqlobject.SQLObject):

    # Fachname.
    name = sqlobject.StringCol(length=255)

    # Joins.
    users = sqlobject.MultipleJoin("user")

class alias(sqlobject.SQLObject):

    # Alias and type.
    alias = sqlobject.StringCol(length=255)
    aliastype = sqlobject.IntCol()

    # User reference.
    user = sqlobject.ForeignKey("user")

class User(object):

    # Initialization methods.

    def __init__(self,userob,name):
        self.__userob = userob
        self.__username = name

    @classmethod
    def getUser(selfcls,name):
        # Split domain part from user name.
        try:
            name, domain = name.split("%",1)
        except:
            raise KeyError, name

        # Check for correct domain.
        if domain <> config.VMAIL_DOMAIN:
            raise KeyError, name

        # Now, try to get user (by mat.nr.)
        try:
            ruser = user.select(user.q.matnr==int(name))[0]
            return selfcls(ruser,str(ruser.matnr))
        except:
            raise KeyError, name

    @classmethod
    def getAlias(selfcls,name):
        # Split domain part from user name.
        try:
            name, domain = name.split("@",1)
        except:
            raise KeyError, name

        # Check for correct domain.
        if domain <> config.VMAIL_DOMAIN:
            raise KeyError, name

        # Strip possible "" from name part.
        name = name.strip("\"")

        # Now, try to get user (by one of his aliases)
        try:
            ruser = alias.select(alias.q.alias==name)[0].user
            return selfcls(ruser,str(ruser.matnr))
        except Exception, e:
            print e, repr(e)
            raise KeyError, name

    @classmethod
    def getUid(selfcls,uid):
        # We don't support getting by Uid for the student authentication.
        # (all users have the same uid, so no sense in this...)
        raise KeyError, uid

    @classmethod
    def listUsers(selfcls):
        # Iterator as classmethod, which returns all users in a list.
        for item in user.select():
            yield selfcls(item,int(item.matnr))

    # PAM Methods.

    def getUsername(self):
        return self.__username

    def getPass(self):
        return self.__userob.passwd

    def checkPass(self,authtok):
        # Check authentication token against database password.
        return self.__userob.passwd == authtok

    def setPass(self,authtok):
        # Set authentication token.
        self.__userob.passwd = authtok

    def openSession(self):
        # TODO: Implement opening of session, create home is in getPasswd(),
        # which tells programs where the home dir is...
        pass

    def closeSession(self):
        # TODO: Implement closing of session.
        pass

    # NSS methods.

    def getPasswd(self):
        # Check user and create home if necessary.
        return Passwd(self.__username,self.__userob.vorname,
                      self.__userob.nachname,self.__userob.matnr,
                      self.__userob.alias)

    def getShadow(self):
        # Return shadow structure.
        return Shadow(self.__username)

    # Squirrelmail methods.

    def getMailInfo(self):
        return ("\"%s, %s\"" % (self.__userob.nachname,self.__userob.vorname),
                "%s@%s" % (self.__userob.alias,config.VMAIL_DOMAIN))

    def setMailInfo(self,alias):
        raise ValueError, "Cannot set alias for now."

class Passwd(pyauthd_nss.Passwd):

    def __init__(self,username,vorname,nachname,matnr,alias):
        super(Passwd,self).__init__("%s%%%s" % (username,config.VMAIL_DOMAIN),
                                    "x",config.VMAIL_UID,config.VMAIL_GID,
                                    "%s %s" % (vorname,nachname),
                                    "%s/%s" % (config.VMAIL_HOME,matnr),
                                    "/bin/false")

        # Create home directory and initial mail. This only works if we are
        # root!
        if not os.path.exists(self.dir):
            # Create home with maildir structure.
            os.system("/bin/cp -r /etc/skel.vmail %s" % self.dir)
            os.system("/usr/bin/chown -R %s:%s %s" %
                      (self.uid,self.gid,self.dir))

            # Send initial mail.
            try:
                host = smtplib.SMTP("localhost")
                host.sendmail("admin@asta.mh-hannover.de",
                              ["%s%%%s" % (username,config.VMAIL_DOMAIN)],
                              config.initmail % {"alias":alias,
                                                 "vorname":vorname,
                                                 "nachname":nachname,
                                                 "matnr":matnr})
            except Exception, e:
                print e, repr(e)

class Shadow(pyauthd_nss.Shadow):

    def __init__(self,username):
        # The default values here are for a user that is always on.
        super(Shadow,self).__init__("%s%%%s" % (username,config.VMAIL_DOMAIN),
                                    "!",13074,0,99999,7,-1,-1,-1)
