/* PyAuthD_NSS_group.c
   --------------------

   Implements the NSS logic for sending/receiving data from the network for
   all group functions.

   Copyright (C) 2005, Heiko Wundram.
   Released under a BSD-style license, see LICENSE for details.

$LastChangedBy: modelnine $
$LastChangedRevision: 75 $
$LastChangedDate: 2005-10-24 01:51:59 +0200 (pon, 24 paź 2005) $
*/

#include <errno.h>
#include <grp.h>
#include <malloc.h>
#include <nss.h>
#include <pwd.h>
#include <shadow.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <syslog.h>
#include <unistd.h>

/* Load specifications for PyAuthD and PyAuthD_NSS. */

#include "PyAuthD_client.h"
#include "PyAuthD_NSS.h"

/* Local socket for getgrent/setgrent/endgrent. */

static int sock = 0;

NSS_STATUS _nss_PyAuthD_getgrgid_r(gid_t gid, struct group *gr, char *buffer,
				   size_t buflen, int *errnop)
{
  int sock, comm = AUTHD_NSS_GETGRGID, vsl, rv;

  if( ( sock = openPyAuthD(AUTHD_NSS) ) == -1 )
    goto SOCKET_ERROR;

  SEND_VALUE(comm);
  SEND_VALUE(gid);
  RECV_VALUE(rv);

  if( rv != NSS_STATUS_SUCCESS ) {
    RECV_VALUE((*errnop));
    return rv;
  } else
    *errnop = 0;

  RECV_GROUP(gr,comm,vsl);

  closePyAuthD(sock);
  return rv;

 MEMORY_ERROR:
  *errnop = ERANGE;
  closePyAuthD(sock);
  return NSS_STATUS_TRYAGAIN;

 SEND_ERROR:
 RECV_ERROR:
 SOCKET_ERROR:
  syslog(LOG_AUTHPRIV|LOG_CRIT,"Unable to send/receive data.");
  *errnop = ENOENT;
  closePyAuthD(sock);
  return NSS_STATUS_UNAVAIL;
}

NSS_STATUS _nss_PyAuthD_getgrnam_r(const char *name, struct group *gr,
				   char *buffer, size_t buflen, int *errnop)
{
  int sock, comm = AUTHD_NSS_GETGRNAM, vsl, rv;

  if( ( sock = openPyAuthD(AUTHD_NSS) ) == -1 )
    goto SOCKET_ERROR;

  SEND_VALUE(comm);
  SEND_DATA(name,comm);
  RECV_VALUE(rv);

  if( rv != NSS_STATUS_SUCCESS ) {
    RECV_VALUE((*errnop));
    return rv;
  } else
    *errnop = 0;

  RECV_GROUP(gr,comm,vsl);

  closePyAuthD(sock);
  return rv;

 MEMORY_ERROR:
  *errnop = ERANGE;
  closePyAuthD(sock);
  return NSS_STATUS_TRYAGAIN;

 SEND_ERROR:
 RECV_ERROR:
 SOCKET_ERROR:
  syslog(LOG_AUTHPRIV|LOG_CRIT,"Unable to send/receive data.");
  *errnop = ENOENT;
  closePyAuthD(sock);
  return NSS_STATUS_UNAVAIL;
}

NSS_STATUS _nss_PyAuthD_setgrent()
{
  int comm = AUTHD_NSS_GETGRENT, rv;

  if( ( sock = openPyAuthD(AUTHD_NSS) ) == -1 )
    goto SOCKET_ERROR;

  SEND_VALUE(comm);
  RECV_VALUE(rv);

  if( rv != NSS_STATUS_SUCCESS ) {
    closePyAuthD(sock);
    sock = 0;
  }

  return rv;

 SEND_ERROR:
 RECV_ERROR:
 SOCKET_ERROR:
  sock = 0;
  return NSS_STATUS_UNAVAIL;
}

NSS_STATUS _nss_PyAuthD_endgrent()
{
  if( sock )
    closePyAuthD(sock);
  sock = 0;
  return NSS_STATUS_SUCCESS;
}

NSS_STATUS _nss_PyAuthD_getgrent_r(struct group *gr, char *buffer,
				   size_t buflen, int *errnop)
{
  int len, vsl, rv;

  if( !sock )
    _nss_PyAuthD_setgrent();
  if( !sock ) {
    /* Second invocation also denied opening socket, just let it be. */
    *errnop = ENOENT;
    return NSS_STATUS_UNAVAIL;
  }

  RECV_VALUE(rv);
  if( rv == NSS_STATUS_SUCCESS )
    RECV_GROUP(gr,len,vsl);

  return rv;

 MEMORY_ERROR:
  /* This handling sucks, and will break stuff, maybe... */
  closePyAuthD(sock);
  sock = 0;
  syslog(LOG_AUTHPRIV|LOG_CRIT,"TODO: Implement MEMORY_ERROR handling!");
  *errnop = ERANGE;
  return NSS_STATUS_TRYAGAIN;

 RECV_ERROR:
  closePyAuthD(sock);
  sock = 0;
  syslog(LOG_AUTHPRIV|LOG_CRIT,"Connection dropped while reading data.");
  *errnop = ENOENT;
  return NSS_STATUS_UNAVAIL;
}
